# 
# Generated by the nebius.base.protos.compiler.  DO NOT EDIT!
# 

"""Auto-generated Nebius SDK package for ``nebius.quotas.v1``"""

import builtins as builtins
import collections.abc as abc
import google.protobuf.descriptor as descriptor_1
import google.protobuf.message as message_1
import nebius.aio.client as client
import nebius.aio.request as request
import nebius.aio.request_kwargs as request_kwargs
import nebius.api.nebius.common.v1 as v1_1
import nebius.api.nebius.common.v1.metadata_pb2 as metadata_pb2
import nebius.api.nebius.quotas.v1.quota_allowance_pb2 as quota_allowance_pb2
import nebius.api.nebius.quotas.v1.quota_allowance_service_pb2 as quota_allowance_service_pb2
import nebius.base.protos.descriptor as descriptor
import nebius.base.protos.pb_classes as pb_classes
import nebius.base.protos.pb_enum as pb_enum
import nebius.base.protos.unset as unset
import typing as typing
import typing_extensions as typing_extensions
#@ local imports here @#

# file: nebius/quotas/v1/quota_allowance.proto
class QuotaAllowanceSpec(pb_classes.Message):
    __PB2_CLASS__ = quota_allowance_pb2.QuotaAllowanceSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.quotas.v1.QuotaAllowanceSpec",quota_allowance_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    class __OneOfClass__limit__(pb_classes.OneOf):
        name: builtins.str= "_limit"
        
        def __init__(self, msg: "QuotaAllowanceSpec") -> None:
            super().__init__()
            self._message: "QuotaAllowanceSpec" = msg
    
    class __OneOfClass__limit_limit__(__OneOfClass__limit__):
        field: typing.Literal["limit"] = "limit"
        
        def __init__(self, msg: "QuotaAllowanceSpec") -> None:
            super().__init__(msg)
        @builtins.property
        def value(self) -> "builtins.int":
            return self._message.limit
    
    @builtins.property
    def _limit(self) -> __OneOfClass__limit_limit__|None:
        field_name_1: str|None = super().which_field_in_oneof("_limit")
        match field_name_1:
            case "limit":
                return self.__OneOfClass__limit_limit__(self)
            case None:
                return None
            case _:
                raise pb_classes.OneOfMatchError(field_name_1)
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        limit: "builtins.int|None|unset.UnsetType" = unset.Unset,
        region: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(limit, unset.UnsetType):
            self.limit = limit
        if not isinstance(region, unset.UnsetType):
            self.region = region
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "limit",
            "region",
            "_limit",
        ]
    
    @builtins.property
    def limit(self) -> "builtins.int|None":
        """
        Total amount of resources allocated.
        """
        
        return super()._get_field("limit", explicit_presence=True,
        )
    @limit.setter
    def limit(self, value: "builtins.int|None") -> None:
        return super()._set_field("limit",value,explicit_presence=True,
        )
    
    @builtins.property
    def region(self) -> "builtins.str":
        """
        Name of the region where the quota is allocated.
        Example: "eu-north1".
        """
        
        return super()._get_field("region", explicit_presence=False,
        )
    @region.setter
    def region(self, value: "builtins.str|None") -> None:
        return super()._set_field("region",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "limit":"limit",
        "region":"region",
        "_limit":"_limit",
    }
    
class QuotaAllowanceStatus(pb_classes.Message):
    __PB2_CLASS__ = quota_allowance_pb2.QuotaAllowanceStatus
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.quotas.v1.QuotaAllowanceStatus",quota_allowance_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    class State(pb_enum.Enum):
        __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.quotas.v1.QuotaAllowanceStatus.State",quota_allowance_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
        STATE_UNSPECIFIED = 0
        STATE_PROVISIONING = 1
        """
        Quota is being allocated; the process can take up to several minutes.
        """
        
        STATE_ACTIVE = 2
        """
        Quota is allocated and can be used.
        """
        
        STATE_FROZEN = 3
        """
        Quota is allocated but cannot be used any longer
        """
        
        STATE_DELETED = 10
        """
        Quota has been removed and is no longer allocated.
        """
        
    
    class UsageState(pb_enum.Enum):
        __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.quotas.v1.QuotaAllowanceStatus.UsageState",quota_allowance_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
        USAGE_STATE_UNSPECIFIED = 0
        USAGE_STATE_USED = 1
        """
        Quota is actively in use.
        """
        
        USAGE_STATE_NOT_USED = 2
        """
        Quota is not currently in use.
        """
        
        USAGE_STATE_UNKNOWN = 3
        """
        Quota region is unreachable, the current usage is therefore unknown.
        Please, retry the request later.
        """
        
        USAGE_STATE_NOT_APPLICABLE = 4
        """
        Quota usage is not applicable
        """
        
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        state: "QuotaAllowanceStatus.State|quota_allowance_pb2.QuotaAllowanceStatus.State|None|unset.UnsetType" = unset.Unset,
        usage: "builtins.int|None|unset.UnsetType" = unset.Unset,
        service: "builtins.str|None|unset.UnsetType" = unset.Unset,
        description: "builtins.str|None|unset.UnsetType" = unset.Unset,
        service_description: "builtins.str|None|unset.UnsetType" = unset.Unset,
        unit: "builtins.str|None|unset.UnsetType" = unset.Unset,
        usage_percentage: "builtins.str|None|unset.UnsetType" = unset.Unset,
        usage_state: "QuotaAllowanceStatus.UsageState|quota_allowance_pb2.QuotaAllowanceStatus.UsageState|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(state, unset.UnsetType):
            self.state = state
        if not isinstance(usage, unset.UnsetType):
            self.usage = usage
        if not isinstance(service, unset.UnsetType):
            self.service = service
        if not isinstance(description, unset.UnsetType):
            self.description = description
        if not isinstance(service_description, unset.UnsetType):
            self.service_description = service_description
        if not isinstance(unit, unset.UnsetType):
            self.unit = unit
        if not isinstance(usage_percentage, unset.UnsetType):
            self.usage_percentage = usage_percentage
        if not isinstance(usage_state, unset.UnsetType):
            self.usage_state = usage_state
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "state",
            "usage",
            "service",
            "description",
            "service_description",
            "unit",
            "usage_percentage",
            "usage_state",
            "State",
            "UsageState",
        ]
    
    @builtins.property
    def state(self) -> "QuotaAllowanceStatus.State":
        """
        Current state of the quota.
        """
        
        return super()._get_field("state", explicit_presence=False,
        wrap=QuotaAllowanceStatus.State,
        )
    @state.setter
    def state(self, value: "QuotaAllowanceStatus.State|quota_allowance_pb2.QuotaAllowanceStatus.State|None") -> None:
        return super()._set_field("state",value,explicit_presence=False,
        )
    
    @builtins.property
    def usage(self) -> "builtins.int":
        """
        Current quota usage.
        """
        
        return super()._get_field("usage", explicit_presence=False,
        )
    @usage.setter
    def usage(self, value: "builtins.int|None") -> None:
        return super()._set_field("usage",value,explicit_presence=False,
        )
    
    @builtins.property
    def service(self) -> "builtins.str":
        """
        Service in which the quota is allocated.
        Example: "mk8s".
        """
        
        return super()._get_field("service", explicit_presence=False,
        )
    @service.setter
    def service(self, value: "builtins.str|None") -> None:
        return super()._set_field("service",value,explicit_presence=False,
        )
    
    @builtins.property
    def description(self) -> "builtins.str":
        """
        Human-readable description of the quota.
        Example: "Total RAM across VMs".
        """
        
        return super()._get_field("description", explicit_presence=False,
        )
    @description.setter
    def description(self, value: "builtins.str|None") -> None:
        return super()._set_field("description",value,explicit_presence=False,
        )
    
    @builtins.property
    def service_description(self) -> "builtins.str":
        """
        Human-readable name of the service managing the quota.
        Example: "Managed Kubernetes®".
        """
        
        return super()._get_field("service_description", explicit_presence=False,
        )
    @service_description.setter
    def service_description(self, value: "builtins.str|None") -> None:
        return super()._set_field("service_description",value,explicit_presence=False,
        )
    
    @builtins.property
    def unit(self) -> "builtins.str":
        """
        Quota unit.
        Example: "byte".
        """
        
        return super()._get_field("unit", explicit_presence=False,
        )
    @unit.setter
    def unit(self, value: "builtins.str|None") -> None:
        return super()._set_field("unit",value,explicit_presence=False,
        )
    
    @builtins.property
    def usage_percentage(self) -> "builtins.str":
        """
        Current quota usage as a percentage.
        Values range from 0.0 to 1.0, representing 0% to 100%.
        Values can exceed 1.0 if usage exceeds the limit.
        Example: "0.12".
        """
        
        return super()._get_field("usage_percentage", explicit_presence=False,
        )
    @usage_percentage.setter
    def usage_percentage(self, value: "builtins.str|None") -> None:
        return super()._set_field("usage_percentage",value,explicit_presence=False,
        )
    
    @builtins.property
    def usage_state(self) -> "QuotaAllowanceStatus.UsageState":
        """
        Current state of the quota usage.
        """
        
        return super()._get_field("usage_state", explicit_presence=False,
        wrap=QuotaAllowanceStatus.UsageState,
        )
    @usage_state.setter
    def usage_state(self, value: "QuotaAllowanceStatus.UsageState|quota_allowance_pb2.QuotaAllowanceStatus.UsageState|None") -> None:
        return super()._set_field("usage_state",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "state":"state",
        "usage":"usage",
        "service":"service",
        "description":"description",
        "service_description":"service_description",
        "unit":"unit",
        "usage_percentage":"usage_percentage",
        "usage_state":"usage_state",
        "State":"State",
        "UsageState":"UsageState",
    }
    
class QuotaAllowance(pb_classes.Message):
    __PB2_CLASS__ = quota_allowance_pb2.QuotaAllowance
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.quotas.v1.QuotaAllowance",quota_allowance_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "QuotaAllowanceSpec|quota_allowance_pb2.QuotaAllowanceSpec|None|unset.UnsetType" = unset.Unset,
        status: "QuotaAllowanceStatus|quota_allowance_pb2.QuotaAllowanceStatus|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
        if not isinstance(status, unset.UnsetType):
            self.status = status
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
            "status",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "QuotaAllowanceSpec":
        return super()._get_field("spec", explicit_presence=False,
        wrap=QuotaAllowanceSpec,
        )
    @spec.setter
    def spec(self, value: "QuotaAllowanceSpec|quota_allowance_pb2.QuotaAllowanceSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    @builtins.property
    def status(self) -> "QuotaAllowanceStatus":
        return super()._get_field("status", explicit_presence=False,
        wrap=QuotaAllowanceStatus,
        )
    @status.setter
    def status(self, value: "QuotaAllowanceStatus|quota_allowance_pb2.QuotaAllowanceStatus|None") -> None:
        return super()._set_field("status",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
        "status":"status",
    }
    
# file: nebius/quotas/v1/quota_allowance_service.proto
class ListQuotaAllowancesRequest(pb_classes.Message):
    __PB2_CLASS__ = quota_allowance_service_pb2.ListQuotaAllowancesRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.quotas.v1.ListQuotaAllowancesRequest",quota_allowance_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        page_size: "builtins.int|None|unset.UnsetType" = unset.Unset,
        page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
        if not isinstance(page_size, unset.UnsetType):
            self.page_size = page_size
        if not isinstance(page_token, unset.UnsetType):
            self.page_token = page_token
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "parent_id",
            "page_size",
            "page_token",
        ]
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        """
        ID of the Container to list quotas for.
        """
        
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_size(self) -> "builtins.int":
        """
        Page size. Must be between [1...1000].
        Optional; if not specified, a reasonable default will be chosen by the service.
        """
        
        return super()._get_field("page_size", explicit_presence=False,
        )
    @page_size.setter
    def page_size(self, value: "builtins.int|None") -> None:
        return super()._set_field("page_size",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_token(self) -> "builtins.str":
        """
        Listing continuation token. Pass an empty string to start listing from the first page.
        """
        
        return super()._get_field("page_token", explicit_presence=False,
        )
    @page_token.setter
    def page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("page_token",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "parent_id":"parent_id",
        "page_size":"page_size",
        "page_token":"page_token",
    }
    
class GetQuotaAllowanceRequest(pb_classes.Message):
    __PB2_CLASS__ = quota_allowance_service_pb2.GetQuotaAllowanceRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.quotas.v1.GetQuotaAllowanceRequest",quota_allowance_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        ID of the quota.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
    }
    
class ListQuotaAllowancesResponse(pb_classes.Message):
    __PB2_CLASS__ = quota_allowance_service_pb2.ListQuotaAllowancesResponse
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.quotas.v1.ListQuotaAllowancesResponse",quota_allowance_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        items: "abc.Iterable[QuotaAllowance]|None|unset.UnsetType" = unset.Unset,
        next_page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(items, unset.UnsetType):
            self.items = items
        if not isinstance(next_page_token, unset.UnsetType):
            self.next_page_token = next_page_token
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "items",
            "next_page_token",
        ]
    
    @builtins.property
    def items(self) -> "abc.MutableSequence[QuotaAllowance]":
        """
        List of quotas on this result page.
        """
        
        return super()._get_field("items", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(QuotaAllowance,None,None),
        )
    @items.setter
    def items(self, value: "abc.Iterable[QuotaAllowance]|None") -> None:
        return super()._set_field("items",value,explicit_presence=False,
        )
    
    @builtins.property
    def next_page_token(self) -> "builtins.str":
        """
        Listing continuation token for the next page of results.
        """
        
        return super()._get_field("next_page_token", explicit_presence=False,
        )
    @next_page_token.setter
    def next_page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("next_page_token",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "items":"items",
        "next_page_token":"next_page_token",
    }
    
class GetByNameRequest(pb_classes.Message):
    __PB2_CLASS__ = quota_allowance_service_pb2.GetByNameRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.quotas.v1.GetByNameRequest",quota_allowance_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
        region: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
        if not isinstance(name, unset.UnsetType):
            self.name = name
        if not isinstance(region, unset.UnsetType):
            self.region = region
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "parent_id",
            "name",
            "region",
        ]
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        """
        ID of the Container to list quotas for.
        """
        
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def name(self) -> "builtins.str":
        """
        Name of the quota.
        Example: "compute.disk.size.network-ssd".
        """
        
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    @builtins.property
    def region(self) -> "builtins.str":
        """
        Name of the region where the quota is allocated.
        Example: "eu-north1".
        """
        
        return super()._get_field("region", explicit_presence=False,
        )
    @region.setter
    def region(self, value: "builtins.str|None") -> None:
        return super()._set_field("region",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "parent_id":"parent_id",
        "name":"name",
        "region":"region",
    }
    

class QuotaAllowanceServiceClient(client.Client):
    """
    This class provides the client methods for the ``.nebius.quotas.v1.QuotaAllowanceService`` service.
    
    Each method constructs a :class:`nebius.aio.request.Request` object
    that represents the in-flight RPC. The request can be awaited (async)
    or waited synchronously using its ``.wait()`` helpers.
    
    The request methods accept various parameters to configure metadata,
    timeouts, authorization, and retries. See individual method docstrings
    for details.
    
    :cvar __service_name__: The full protobuf service name.
    """
    
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.ServiceDescriptor](".nebius.quotas.v1.QuotaAllowanceService",quota_allowance_service_pb2.DESCRIPTOR,descriptor_1.ServiceDescriptor)
    """The protobuf service descriptor extraction function."""
    __service_name__ = ".nebius.quotas.v1.QuotaAllowanceService"
    
    def list(self,
        request: "ListQuotaAllowancesRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request.Request["ListQuotaAllowancesRequest","ListQuotaAllowancesResponse"]:
        """
        Lists quotas by an ID of a Tenant or a Project.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.quotas.v1.ListQuotaAllowancesRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.quotas.v1.ListQuotaAllowancesResponse`.
        """
        
        return super().request(
            method="List",
            request=request,
            result_pb2_class=quota_allowance_service_pb2.ListQuotaAllowancesResponse,
            result_wrapper=pb_classes.simple_wrapper(ListQuotaAllowancesResponse),
            **kwargs,
        )
    
    def get(self,
        request: "GetQuotaAllowanceRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request.Request["GetQuotaAllowanceRequest","QuotaAllowance"]:
        """
        Gets a quota by its ID.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.quotas.v1.GetQuotaAllowanceRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.quotas.v1.QuotaAllowance`.
        """
        
        return super().request(
            method="Get",
            request=request,
            result_pb2_class=quota_allowance_pb2.QuotaAllowance,
            result_wrapper=pb_classes.simple_wrapper(QuotaAllowance),
            **kwargs,
        )
    
    def get_by_name(self,
        request: "GetByNameRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request.Request["GetByNameRequest","QuotaAllowance"]:
        """
        Gets a quota by an ID of a Tenant or a Project, its region, and name.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.quotas.v1.GetByNameRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.quotas.v1.QuotaAllowance`.
        """
        
        return super().request(
            method="GetByName",
            request=request,
            result_pb2_class=quota_allowance_pb2.QuotaAllowance,
            result_wrapper=pb_classes.simple_wrapper(QuotaAllowance),
            **kwargs,
        )
    

__all__ = [
    #@ local import names here @#
    "QuotaAllowanceSpec",
    "QuotaAllowanceStatus",
    "QuotaAllowance",
    "ListQuotaAllowancesRequest",
    "GetQuotaAllowanceRequest",
    "ListQuotaAllowancesResponse",
    "GetByNameRequest",
    "QuotaAllowanceServiceClient",
]



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class K8sRelease(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: K8sReleaseSpec
    status: K8sReleaseStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[K8sReleaseSpec, _Mapping]] = ..., status: _Optional[_Union[K8sReleaseStatus, _Mapping]] = ...) -> None: ...

class K8sReleaseSpec(_message.Message):
    __slots__ = ["cluster_id", "product_slug", "namespace", "application_name", "values", "set"]
    class SetEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    CLUSTER_ID_FIELD_NUMBER: _ClassVar[int]
    PRODUCT_SLUG_FIELD_NUMBER: _ClassVar[int]
    NAMESPACE_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_NAME_FIELD_NUMBER: _ClassVar[int]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    SET_FIELD_NUMBER: _ClassVar[int]
    cluster_id: str
    product_slug: str
    namespace: str
    application_name: str
    values: str
    set: _containers.ScalarMap[str, str]
    def __init__(self, cluster_id: _Optional[str] = ..., product_slug: _Optional[str] = ..., namespace: _Optional[str] = ..., application_name: _Optional[str] = ..., values: _Optional[str] = ..., set: _Optional[_Mapping[str, str]] = ...) -> None: ...

class K8sReleaseStatus(_message.Message):
    __slots__ = ["state", "error_message"]
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        UNSPECIFIED: _ClassVar[K8sReleaseStatus.State]
        CREATED: _ClassVar[K8sReleaseStatus.State]
        RUNNING: _ClassVar[K8sReleaseStatus.State]
        DEPLOYED: _ClassVar[K8sReleaseStatus.State]
        FAILED: _ClassVar[K8sReleaseStatus.State]
        INSTALLING: _ClassVar[K8sReleaseStatus.State]
    UNSPECIFIED: K8sReleaseStatus.State
    CREATED: K8sReleaseStatus.State
    RUNNING: K8sReleaseStatus.State
    DEPLOYED: K8sReleaseStatus.State
    FAILED: K8sReleaseStatus.State
    INSTALLING: K8sReleaseStatus.State
    STATE_FIELD_NUMBER: _ClassVar[int]
    ERROR_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    state: K8sReleaseStatus.State
    error_message: str
    def __init__(self, state: _Optional[_Union[K8sReleaseStatus.State, str]] = ..., error_message: _Optional[str] = ...) -> None: ...



# 
# Generated by the nebius.base.protos.compiler.  DO NOT EDIT!
# 

"""Auto-generated Nebius SDK package for ``nebius``"""

import builtins as builtins
import collections.abc as abc
import google.protobuf.descriptor as descriptor_1
import google.protobuf.message as message
import nebius.api.nebius.annotations_pb2 as annotations_pb2
import nebius.base.protos.descriptor as descriptor
import nebius.base.protos.pb_classes as pb_classes
import nebius.base.protos.pb_enum as pb_enum
import nebius.base.protos.unset as unset
#@ local imports here @#

# file: nebius/annotations.proto
class ResourceBehavior(pb_enum.Enum):
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.ResourceBehavior",annotations_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
    RESOURCE_BEHAVIOR_UNSPECIFIED = 0
    """
    The behavior of the resource is unspecified.
    Avoid using this default value.
    """
    
    MOVABLE = 1
    """
    Indicates that the resource can be moved to another parent, typically an
    IAM container, though not necessarily limited to this.
    This behavior suggests that the ``metadata.parent_id`` attribute could be modified.
    """
    
    UNNAMED = 2
    """
    Indicates that the resource name can be unspecified or does not follow
    uniqueness requirement within parent_id and resource type.
    """
    
    IMMUTABLE_NAME = 3
    """
    Indicates that the resource is named, and the name cannot be changed after
    it is created. It is strongly recommended to do srvices with renaming
    capability, as the guidelines suggest.
    """
    

class FieldBehavior(pb_enum.Enum):
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.FieldBehavior",annotations_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
    FIELD_BEHAVIOR_UNSPECIFIED = 0
    IMMUTABLE = 2
    """
    This indicates that the field can't be changed during a resource update.
    Changing the field value will cause an ``INVALID_ARGUMENT`` error.
    Resource recreate requires a change of the field value.
    """
    
    IDENTIFIER = 3
    """
    Indicates field is a resource ID, so it MUST be present on a resource
    update, but MUST NOT be set on create.
    Otherwise, RPC will fail with the ``INVALID_ARGUMENT`` error
    """
    
    INPUT_ONLY = 4
    """
    Indicates field is not present in output.
    """
    
    OUTPUT_ONLY = 5
    """
    Indicates field can't be set on create or changed on update.
    Otherwise, RPC will fail with the ``INVALID_ARGUMENT`` error
    """
    
    MEANINGFUL_EMPTY_VALUE = 6
    """
    Indicates that an empty message and a null have different semantics.
    Usually, that field is a feature spec message: its empty message enables
    that feature, and null disables it. Such a message is different from ``bool``
    because it already has some feature parameters, or they can be added later
    in a backward-compatible way.
    IMPORTANT: if the message itself is recursive, this behavior is forced.
    """
    
    NON_EMPTY_DEFAULT = 7
    """
    Indicates that an empty (default) value will be filled by the server.
    Usually, that field is a feature spec value, which by default is computed.
    Values marked with this annotation won't raise error if they are not set
    and the returned value is not equal to protobuf default.
    
    IMPORTANT:
    Updating this value from explicit to default may not lead to Update call in
    some tools (eg Terraform).
    Compound values (messages, lists and maps) may result in unpredictable
    updates (see examples in guidelines).
    """
    

class ServicePySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.ServicePySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.ServicePySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class MethodPySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.MethodPySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.MethodPySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class FieldPySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.FieldPySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.FieldPySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class MessagePySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.MessagePySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.MessagePySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class OneofPySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.OneofPySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.OneofPySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class EnumPySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.EnumPySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.EnumPySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class EnumValuePySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.EnumValuePySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.EnumValuePySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class DeprecationDetails(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.DeprecationDetails
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.DeprecationDetails",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        effective_at: "builtins.str|None|unset.UnsetType" = unset.Unset,
        description: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(effective_at, unset.UnsetType):
            self.effective_at = effective_at
        if not isinstance(description, unset.UnsetType):
            self.description = description
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "effective_at",
            "description",
        ]
    
    @builtins.property
    def effective_at(self) -> "builtins.str":
        """
        The date when this method, service, message or field will stop working (format: YYYY-MM-DD)
        """
        
        return super()._get_field("effective_at", explicit_presence=False,
        )
    @effective_at.setter
    def effective_at(self, value: "builtins.str|None") -> None:
        return super()._set_field("effective_at",value,explicit_presence=False,
        )
    
    @builtins.property
    def description(self) -> "builtins.str":
        """
        A description to help users understand the reason for deprecation and suggest alternatives
        """
        
        return super()._get_field("description", explicit_presence=False,
        )
    @description.setter
    def description(self, value: "builtins.str|None") -> None:
        return super()._set_field("description",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "effective_at":"effective_at",
        "description":"description",
    }
    
class NIDFieldSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.NIDFieldSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.NIDFieldSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        resource: "abc.Iterable[builtins.str]|None|unset.UnsetType" = unset.Unset,
        parent_resource: "abc.Iterable[builtins.str]|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(resource, unset.UnsetType):
            self.resource = resource
        if not isinstance(parent_resource, unset.UnsetType):
            self.parent_resource = parent_resource
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "resource",
            "parent_resource",
        ]
    
    @builtins.property
    def resource(self) -> "abc.MutableSequence[builtins.str]":
        """
        Fields annotated with this option are treated as NIDs.
        ``resource`` lists allowed NID resource types (prefixes). Leave empty to accept any type.
        Validation only produces warnings.
        """
        
        return super()._get_field("resource", explicit_presence=False,
        wrap=pb_classes.Repeated,
        )
    @resource.setter
    def resource(self, value: "abc.Iterable[builtins.str]|None") -> None:
        return super()._set_field("resource",value,explicit_presence=False,
        )
    
    @builtins.property
    def parent_resource(self) -> "abc.MutableSequence[builtins.str]":
        """
        For metadata fields, ``parent_resource`` lists allowed parent resource types for ``metadata.parent_id``.
        Leave empty to allow any type. Validation only produces warnings.
        Typically set on the resource message; request-level overrides are supported.
        """
        
        return super()._get_field("parent_resource", explicit_presence=False,
        wrap=pb_classes.Repeated,
        )
    @parent_resource.setter
    def parent_resource(self, value: "abc.Iterable[builtins.str]|None") -> None:
        return super()._set_field("parent_resource",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "resource":"resource",
        "parent_resource":"parent_resource",
    }
    
file_deprecation_details = annotations_pb2.file_deprecation_details
api_service_name = annotations_pb2.api_service_name
service_deprecation_details = annotations_pb2.service_deprecation_details
service_py_sdk = annotations_pb2.service_py_sdk
method_deprecation_details = annotations_pb2.method_deprecation_details
method_py_sdk = annotations_pb2.method_py_sdk
resource_behavior = annotations_pb2.resource_behavior
message_deprecation_details = annotations_pb2.message_deprecation_details
message_py_sdk = annotations_pb2.message_py_sdk
field_behavior = annotations_pb2.field_behavior
sensitive = annotations_pb2.sensitive
credentials = annotations_pb2.credentials
field_deprecation_details = annotations_pb2.field_deprecation_details
field_py_sdk = annotations_pb2.field_py_sdk
nid = annotations_pb2.nid
oneof_behavior = annotations_pb2.oneof_behavior
oneof_py_sdk = annotations_pb2.oneof_py_sdk
enum_py_sdk = annotations_pb2.enum_py_sdk
enum_deprecation_details = annotations_pb2.enum_deprecation_details
enum_value_deprecation_details = annotations_pb2.enum_value_deprecation_details
enum_value_py_sdk = annotations_pb2.enum_value_py_sdk
__all__ = [
    #@ local import names here @#
    "ResourceBehavior",
    "FieldBehavior",
    "file_deprecation_details",
    "api_service_name",
    "service_deprecation_details",
    "service_py_sdk",
    "method_deprecation_details",
    "method_py_sdk",
    "resource_behavior",
    "message_deprecation_details",
    "message_py_sdk",
    "field_behavior",
    "sensitive",
    "credentials",
    "field_deprecation_details",
    "field_py_sdk",
    "nid",
    "oneof_behavior",
    "oneof_py_sdk",
    "enum_py_sdk",
    "enum_deprecation_details",
    "enum_value_deprecation_details",
    "enum_value_py_sdk",
    "ServicePySDKSettings",
    "MethodPySDKSettings",
    "FieldPySDKSettings",
    "MessagePySDKSettings",
    "OneofPySDKSettings",
    "EnumPySDKSettings",
    "EnumValuePySDKSettings",
    "DeprecationDetails",
    "NIDFieldSettings",
]



"""Base client classes used by generated SDK clients.

This module contains small, reusable base classes that concrete generated
clients inherit from. The classes provide a thin wrapper around a
``ClientChannelInterface`` and a convenience ``request`` factory used by the
generated RPC methods.

The types in this module are intentionally lightweight and are not meant to
be instantiated directly by application code; instead they provide a common
shaping layer for code generated from service definitions.
"""

from collections.abc import Callable
from logging import getLogger
from typing import Any, Generic, TypeVar

from google.protobuf.message import Message as PMessage
from typing_extensions import Unpack

from nebius.aio.abc import ClientChannelInterface as Channel
from nebius.aio.constant_channel import Constant
from nebius.aio.request import Request

# from nebius.api.nebius.common.v1 import Operation
from nebius.aio.request_kwargs import RequestKwargs

Req = TypeVar("Req")
Res = TypeVar("Res")


class Client:
    """Lightweight base class for generated service clients.

    Subclasses generated for each service should set the ``__service_name__``
    class attribute and expose RPC methods that in turn call
    :meth:`request` to construct a :class:`nebius.aio.request.Request`.

    :cvar __service_name__: the fully-qualified service name used in RPC
        routing (string).
    :cvar __service_deprecation_details__: optional deprecation message
        emitted as a runtime warning when the client is constructed.

    :param channel: a channel implementing :class:`ClientChannelInterface`
    :type channel: :class:`ClientChannelInterface`
    """

    # __operation_type__: Message = Operation
    __service_name__: str
    __service_deprecation_details__: str | None = None

    def __init__(self, channel: Channel) -> None:
        """Create a client bound to a channel."""
        self._channel = channel

        if self.__service_deprecation_details__ is not None:
            getLogger("deprecation").warning(
                f"Service {self.__service_name__} is deprecated. "
                f"{self.__service_deprecation_details__}",
                stack_info=True,
                stacklevel=2,
            )

    def request(
        self,
        method: str,
        request: Req,
        result_pb2_class: type[PMessage],
        result_wrapper: Callable[[str, Channel, Any], Res] | None = None,
        **kwargs: Unpack[RequestKwargs],
    ) -> Request[Req, Res]:
        """Construct a :class:`nebius.aio.request.Request` for an RPC.

        Subclasses' generated RPC methods call this helper to create a
        Request object with the appropriate service/method names and options.

        :param method: RPC method name (bare, without service prefix)
        :type method: `str`
        :param request: protobuf message or request payload accepted by the RPC
        :param result_pb2_class: protobuf class of the RPC response message
        :type result_pb2_class: type of the protobuf result message
        :param result_wrapper: optional callable to post-process the RPC result

        Other keyword arguments are passed through to the
        :class:`nebius.aio.request.Request` constructor.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.

        :returns: a configured :class:`nebius.aio.request.Request` instance
        :rtype: :class:`Request` of the return type of the RPC or the result of
            ``result_wrapper`` if provided.
        """
        return Request[Req, Res](
            channel=self._channel,
            service=self.__service_name__,
            method=method,
            request=request,
            result_pb2_class=result_pb2_class,
            result_wrapper=result_wrapper,
            **kwargs,
        )


OperationPb = TypeVar("OperationPb")
OperationService = TypeVar("OperationService", bound=Client)


class ClientWithOperations(Client, Generic[OperationPb, OperationService]):
    """Extension of :class:`Client` for services that manage long-running
    operations.

    This helper provides an :meth:`operation_service` accessor that lazily creates
    and caches a small helper client bound to a synthetic constant channel
    that targets the service's operation methods.

    :cvar __operation_type__: the protobuf message class used to represent
        long-running operations.
    :cvar __operation_service_class__: the client class used to manage
        operations.
    :cvar __operation_source_method__: the method name used to identify
        the source of operations for this service (for example, "CreateFoo"
        if the service's "CreateFoo" method returns operations).
    :ivar __operation_service__: cached instance of the operation-service client.

    :param channel: channel used for normal RPCs; a special constant
        channel will be created for the operation service when needed.
    """

    __operation_type__: type[OperationPb]
    __operation_service_class__: type[OperationService]
    __operation_source_method__: str

    def __init__(self, channel: Channel) -> None:
        """Initialize the client-with-operations."""
        super().__init__(channel)
        self.__operation_service__: OperationService | None = None

    def operation_service(self) -> OperationService:
        """Return a cached operation-service client instance.

        The operation-service client is created on first access and cached on
        the instance. The created client is an instance of the
        ``__operation_service_class__`` and is bound to a
        :class:`nebius.aio.constant_channel.Constant` that routes calls to the
        service's operation endpoint.

        :returns: an instance of the operation service client
        :rtype: OperationService
        """
        if self.__operation_service__ is None:
            self.__operation_service__ = self.__operation_service_class__(
                Constant(
                    self.__service_name__ + "." + self.__operation_source_method__,
                    self._channel,
                )
            )
        return self.__operation_service__



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/storage/v1/lifecycle.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius.storage.v1 import base_pb2 as nebius_dot_storage_dot_v1_dot_base__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!nebius/storage/v1/lifecycle.proto\x12\x11nebius.storage.v1\x1a\x1cnebius/storage/v1/base.proto\x1a\x18nebius/annotations.proto\x1a\x1b\x62uf/validate/validate.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xb3\x01\n\x16LifecycleConfiguration\x12\x41\n\x05rules\x18\x01 \x03(\x0b\x32 .nebius.storage.v1.LifecycleRuleB\t\xbaH\x06\x92\x01\x03\x10\xe8\x07R\x05rules\x12V\n\x12last_access_filter\x18\x02 \x01(\x0b\x32(.nebius.storage.v1.LifecycleAccessFilterR\x10lastAccessFilter\"\x81\x06\n\rLifecycleRule\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\x12G\n\x06status\x18\x02 \x01(\x0e\x32\'.nebius.storage.v1.LifecycleRule.StatusB\x06\xbaH\x03\xc8\x01\x01R\x06status\x12@\n\x06\x66ilter\x18\x03 \x01(\x0b\x32\".nebius.storage.v1.LifecycleFilterB\x04\xbaJ\x01\x07R\x06\x66ilter\x12\x46\n\nexpiration\x18\x04 \x01(\x0b\x32&.nebius.storage.v1.LifecycleExpirationR\nexpiration\x12{\n\x1dnoncurrent_version_expiration\x18\x05 \x01(\x0b\x32\x37.nebius.storage.v1.LifecycleNoncurrentVersionExpirationR\x1bnoncurrentVersionExpiration\x12\x85\x01\n!abort_incomplete_multipart_upload\x18\x06 \x01(\x0b\x32:.nebius.storage.v1.LifecycleAbortIncompleteMultipartUploadR\x1e\x61\x62ortIncompleteMultipartUpload\x12\x46\n\ntransition\x18\x07 \x01(\x0b\x32&.nebius.storage.v1.LifecycleTransitionR\ntransition\x12{\n\x1dnoncurrent_version_transition\x18\x08 \x01(\x0b\x32\x37.nebius.storage.v1.LifecycleNoncurrentVersionTransitionR\x1bnoncurrentVersionTransition\";\n\x06Status\x12\x16\n\x12STATUS_UNSPECIFIED\x10\x00\x12\x0b\n\x07\x45NABLED\x10\x01\x12\x0c\n\x08\x44ISABLED\x10\x02\"\xab\x01\n\x0fLifecycleFilter\x12\x16\n\x06prefix\x18\x01 \x01(\tR\x06prefix\x12\x42\n\x1eobject_size_greater_than_bytes\x18\x02 \x01(\x03R\x1aobjectSizeGreaterThanBytes\x12<\n\x1bobject_size_less_than_bytes\x18\x03 \x01(\x03R\x17objectSizeLessThanBytes\"\x9d\x04\n\x15LifecycleAccessFilter\x12\\\n\nconditions\x18\x01 \x03(\x0b\x32\x32.nebius.storage.v1.LifecycleAccessFilter.ConditionB\x08\xbaH\x05\x92\x01\x02\x10\x64R\nconditions\x1a\xa5\x03\n\tCondition\x12S\n\x04type\x18\x01 \x01(\x0e\x32\x37.nebius.storage.v1.LifecycleAccessFilter.Condition.TypeB\x06\xbaH\x03\xc8\x01\x01R\x04type\x12]\n\x07methods\x18\x02 \x03(\x0e\x32\x39.nebius.storage.v1.LifecycleAccessFilter.Condition.MethodB\x08\xbaH\x05\x92\x01\x02\x10\nR\x07methods\x12)\n\x0buser_agents\x18\x03 \x03(\tB\x08\xbaH\x05\x92\x01\x02\x10\x64R\nuserAgents\"6\n\x04Type\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x0b\n\x07INCLUDE\x10\x01\x12\x0b\n\x07\x45XCLUDE\x10\x02\"\x80\x01\n\x06Method\x12\x16\n\x12METHOD_UNSPECIFIED\x10\x00\x12\x0e\n\nGET_OBJECT\x10\x01\x12\x0f\n\x0bHEAD_OBJECT\x10\x02\x12\x16\n\x12GET_OBJECT_TAGGING\x10\x03\x12\x0f\n\x0b\x43OPY_OBJECT\x10\x04\x12\x14\n\x10UPLOAD_PART_COPY\x10\x05\"\xbb\x01\n\x13LifecycleExpiration\x12\x30\n\x04\x64\x61te\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\x04\x64\x61te\x12!\n\x04\x64\x61ys\x18\x02 \x01(\x05\x42\x0b\xbaH\x08\x1a\x06\x18\xca\x8e\x01(\x01H\x00R\x04\x64\x61ys\x12?\n\x1c\x65xpired_object_delete_marker\x18\x03 \x01(\x08R\x19\x65xpiredObjectDeleteMarkerB\x0e\n\x0c\x65xpired_with\"\xbb\x01\n$LifecycleNoncurrentVersionExpiration\x12?\n\x19newer_noncurrent_versions\x18\x01 \x01(\x05H\x00R\x17newerNoncurrentVersions\x88\x01\x01\x12\x34\n\x0fnoncurrent_days\x18\x02 \x01(\x05\x42\x0b\xbaH\x08\x1a\x06\x18\xca\x8e\x01(\x01R\x0enoncurrentDaysB\x1c\n\x1a_newer_noncurrent_versions\"j\n\'LifecycleAbortIncompleteMultipartUpload\x12?\n\x15\x64\x61ys_after_initiation\x18\x01 \x01(\x05\x42\x0b\xbaH\x08\x1a\x06\x18\xca\x8e\x01(\x01R\x13\x64\x61ysAfterInitiation\"\x86\x02\n\x13LifecycleTransition\x12\x30\n\x04\x64\x61te\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\x04\x64\x61te\x12!\n\x04\x64\x61ys\x18\x02 \x01(\x05\x42\x0b\xbaH\x08\x1a\x06\x18\xca\x8e\x01(\x01H\x00R\x04\x64\x61ys\x12\x42\n\x16\x64\x61ys_since_last_access\x18\x04 \x01(\x05\x42\x0b\xbaH\x08\x1a\x06\x18\xca\x8e\x01(\x01H\x00R\x13\x64\x61ysSinceLastAccess\x12\x44\n\rstorage_class\x18\x03 \x01(\x0e\x32\x1f.nebius.storage.v1.StorageClassR\x0cstorageClassB\x10\n\x0etransited_with\"\x81\x02\n$LifecycleNoncurrentVersionTransition\x12?\n\x19newer_noncurrent_versions\x18\x01 \x01(\x05H\x00R\x17newerNoncurrentVersions\x88\x01\x01\x12\x34\n\x0fnoncurrent_days\x18\x02 \x01(\x05\x42\x0b\xbaH\x08\x1a\x06\x18\xca\x8e\x01(\x01R\x0enoncurrentDays\x12\x44\n\rstorage_class\x18\x03 \x01(\x0e\x32\x1f.nebius.storage.v1.StorageClassR\x0cstorageClassB\x1c\n\x1a_newer_noncurrent_versionsB]\n\x18\x61i.nebius.pub.storage.v1B\x0eLifecycleProtoP\x01Z/github.com/nebius/gosdk/proto/nebius/storage/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.storage.v1.lifecycle_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030ai.nebius.pub.storage.v1B\016LifecycleProtoP\001Z/github.com/nebius/gosdk/proto/nebius/storage/v1'
  _LIFECYCLECONFIGURATION.fields_by_name['rules']._options = None
  _LIFECYCLECONFIGURATION.fields_by_name['rules']._serialized_options = b'\272H\006\222\001\003\020\350\007'
  _LIFECYCLERULE.fields_by_name['id']._options = None
  _LIFECYCLERULE.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _LIFECYCLERULE.fields_by_name['status']._options = None
  _LIFECYCLERULE.fields_by_name['status']._serialized_options = b'\272H\003\310\001\001'
  _LIFECYCLERULE.fields_by_name['filter']._options = None
  _LIFECYCLERULE.fields_by_name['filter']._serialized_options = b'\272J\001\007'
  _LIFECYCLEACCESSFILTER_CONDITION.fields_by_name['type']._options = None
  _LIFECYCLEACCESSFILTER_CONDITION.fields_by_name['type']._serialized_options = b'\272H\003\310\001\001'
  _LIFECYCLEACCESSFILTER_CONDITION.fields_by_name['methods']._options = None
  _LIFECYCLEACCESSFILTER_CONDITION.fields_by_name['methods']._serialized_options = b'\272H\005\222\001\002\020\n'
  _LIFECYCLEACCESSFILTER_CONDITION.fields_by_name['user_agents']._options = None
  _LIFECYCLEACCESSFILTER_CONDITION.fields_by_name['user_agents']._serialized_options = b'\272H\005\222\001\002\020d'
  _LIFECYCLEACCESSFILTER.fields_by_name['conditions']._options = None
  _LIFECYCLEACCESSFILTER.fields_by_name['conditions']._serialized_options = b'\272H\005\222\001\002\020d'
  _LIFECYCLEEXPIRATION.fields_by_name['days']._options = None
  _LIFECYCLEEXPIRATION.fields_by_name['days']._serialized_options = b'\272H\010\032\006\030\312\216\001(\001'
  _LIFECYCLENONCURRENTVERSIONEXPIRATION.fields_by_name['noncurrent_days']._options = None
  _LIFECYCLENONCURRENTVERSIONEXPIRATION.fields_by_name['noncurrent_days']._serialized_options = b'\272H\010\032\006\030\312\216\001(\001'
  _LIFECYCLEABORTINCOMPLETEMULTIPARTUPLOAD.fields_by_name['days_after_initiation']._options = None
  _LIFECYCLEABORTINCOMPLETEMULTIPARTUPLOAD.fields_by_name['days_after_initiation']._serialized_options = b'\272H\010\032\006\030\312\216\001(\001'
  _LIFECYCLETRANSITION.fields_by_name['days']._options = None
  _LIFECYCLETRANSITION.fields_by_name['days']._serialized_options = b'\272H\010\032\006\030\312\216\001(\001'
  _LIFECYCLETRANSITION.fields_by_name['days_since_last_access']._options = None
  _LIFECYCLETRANSITION.fields_by_name['days_since_last_access']._serialized_options = b'\272H\010\032\006\030\312\216\001(\001'
  _LIFECYCLENONCURRENTVERSIONTRANSITION.fields_by_name['noncurrent_days']._options = None
  _LIFECYCLENONCURRENTVERSIONTRANSITION.fields_by_name['noncurrent_days']._serialized_options = b'\272H\010\032\006\030\312\216\001(\001'
  _globals['_LIFECYCLECONFIGURATION']._serialized_start=175
  _globals['_LIFECYCLECONFIGURATION']._serialized_end=354
  _globals['_LIFECYCLERULE']._serialized_start=357
  _globals['_LIFECYCLERULE']._serialized_end=1126
  _globals['_LIFECYCLERULE_STATUS']._serialized_start=1067
  _globals['_LIFECYCLERULE_STATUS']._serialized_end=1126
  _globals['_LIFECYCLEFILTER']._serialized_start=1129
  _globals['_LIFECYCLEFILTER']._serialized_end=1300
  _globals['_LIFECYCLEACCESSFILTER']._serialized_start=1303
  _globals['_LIFECYCLEACCESSFILTER']._serialized_end=1844
  _globals['_LIFECYCLEACCESSFILTER_CONDITION']._serialized_start=1423
  _globals['_LIFECYCLEACCESSFILTER_CONDITION']._serialized_end=1844
  _globals['_LIFECYCLEACCESSFILTER_CONDITION_TYPE']._serialized_start=1659
  _globals['_LIFECYCLEACCESSFILTER_CONDITION_TYPE']._serialized_end=1713
  _globals['_LIFECYCLEACCESSFILTER_CONDITION_METHOD']._serialized_start=1716
  _globals['_LIFECYCLEACCESSFILTER_CONDITION_METHOD']._serialized_end=1844
  _globals['_LIFECYCLEEXPIRATION']._serialized_start=1847
  _globals['_LIFECYCLEEXPIRATION']._serialized_end=2034
  _globals['_LIFECYCLENONCURRENTVERSIONEXPIRATION']._serialized_start=2037
  _globals['_LIFECYCLENONCURRENTVERSIONEXPIRATION']._serialized_end=2224
  _globals['_LIFECYCLEABORTINCOMPLETEMULTIPARTUPLOAD']._serialized_start=2226
  _globals['_LIFECYCLEABORTINCOMPLETEMULTIPARTUPLOAD']._serialized_end=2332
  _globals['_LIFECYCLETRANSITION']._serialized_start=2335
  _globals['_LIFECYCLETRANSITION']._serialized_end=2597
  _globals['_LIFECYCLENONCURRENTVERSIONTRANSITION']._serialized_start=2600
  _globals['_LIFECYCLENONCURRENTVERSIONTRANSITION']._serialized_end=2857
# @@protoc_insertion_point(module_scope)
